package clime.messadmin.core;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.servlet.http.HttpSessionActivationListener;
import javax.servlet.http.HttpSessionEvent;
import javax.servlet.http.HttpSessionListener;

import clime.messadmin.model.Server;
import clime.messadmin.providers.ProviderUtils;
import clime.messadmin.providers.spi.ApplicationLifeCycleProvider;
import clime.messadmin.providers.spi.SessionLifeCycleProvider;

/**
 * Tracks HttpSessions, taking care of active and passive ones.
 * Note: this is a stateless object.
 * @author C&eacute;drik LIME
 */
public class MessAdminListener implements HttpSessionListener, HttpSessionActivationListener, ServletContextListener {

	/**
	 * 
	 */
	public MessAdminListener() {
		super();
	}

	/* HttpSession-related events */

	/**
	 * {@inheritDoc}
	 */
	public void sessionCreated(final HttpSessionEvent event) {
		Server.getInstance().getApplication(event.getSession().getServletContext()).sessionCreated(event);
		Iterator iter = ProviderUtils.getProviders(SessionLifeCycleProvider.class).iterator();
		while (iter.hasNext()) {
			SessionLifeCycleProvider lc = (SessionLifeCycleProvider) iter.next();
			try {
				lc.sessionCreated(event.getSession());
			} catch (RuntimeException rte) {
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void sessionDestroyed(final HttpSessionEvent event) {
		List providers = new ArrayList(ProviderUtils.getProviders(SessionLifeCycleProvider.class));
		Collections.reverse(providers);
		Iterator iter = providers.iterator();
		while (iter.hasNext()) {
			SessionLifeCycleProvider lc = (SessionLifeCycleProvider) iter.next();
			try {
				lc.sessionDestroyed(event.getSession());
			} catch (RuntimeException rte) {
			}
		}
		Server.getInstance().getApplication(event.getSession().getServletContext()).sessionDestroyed(event);
	}

	/**
	 * {@inheritDoc}
	 */
	public void sessionWillPassivate(final HttpSessionEvent se) {
		List providers = new ArrayList(ProviderUtils.getProviders(SessionLifeCycleProvider.class));
		Collections.reverse(providers);
		Iterator iter = providers.iterator();
		while (iter.hasNext()) {
			SessionLifeCycleProvider lc = (SessionLifeCycleProvider) iter.next();
			try {
				lc.sessionWillPassivate(se.getSession());
			} catch (RuntimeException rte) {
			}
		}
		Server.getInstance().getApplication(se.getSession().getServletContext()).sessionWillPassivate(se);
	}

	/**
	 * {@inheritDoc}
	 */
	public void sessionDidActivate(final HttpSessionEvent se) {
		Server.getInstance().getApplication(se.getSession().getServletContext()).sessionDidActivate(se);
		Iterator iter = ProviderUtils.getProviders(SessionLifeCycleProvider.class).iterator();
		while (iter.hasNext()) {
			SessionLifeCycleProvider lc = (SessionLifeCycleProvider) iter.next();
			try {
				lc.sessionDidActivate(se.getSession());
			} catch (RuntimeException rte) {
			}
		}
	}

	/* ServletContext-related events */

	/**
	 * {@inheritDoc}
	 */
	public void contextInitialized(final ServletContextEvent sce) {
		// since we don't have hooks for Server lifecycle, try
		serverInitialized();
		Server.getInstance().contextInitialized(sce);
		Iterator iter = ProviderUtils.getProviders(ApplicationLifeCycleProvider.class).iterator();
		while (iter.hasNext()) {
			ApplicationLifeCycleProvider lc = (ApplicationLifeCycleProvider) iter.next();
			try {
				lc.contextInitialized(sce.getServletContext());
			} catch (RuntimeException rte) {
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void contextDestroyed(final ServletContextEvent sce) {
		List providers = new ArrayList(ProviderUtils.getProviders(ApplicationLifeCycleProvider.class));
		Collections.reverse(providers);
		Iterator iter = providers.iterator();
		while (iter.hasNext()) {
			ApplicationLifeCycleProvider lc = (ApplicationLifeCycleProvider) iter.next();
			try {
				lc.contextDestroyed(sce.getServletContext());
			} catch (RuntimeException rte) {
			}
		}
		Server.getInstance().contextDestroyed(sce);
		ProviderUtils.deregisterCurrent();
		// since we don't have hooks for Server lifecycle, try
		serverDestroyed();
	}

	/* Server-related events */

	/**
	 */
	public void serverInitialized() {
		Server.getInstance().serverInitialized(); // all work is done there
	}

	/**
	 */
	public void serverDestroyed() {
		Server.getInstance().serverDestroyed(); // all work is done there
	}

}
