package clime.messadmin.filter;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

/**
 * Used for counting resquest size
 * @author C&eacute;drik LIME
 */
public class MessAdminRequestWrapper extends HttpServletRequestWrapper {
	protected long requestLength = -1;
	private BufferedReader decryptedDataReader;
	private ServletInputStream decryptedDataInputStream;

	class DelegatingServletInputStream extends ServletInputStream {
		private final InputStream sourceStream;

		/**
		 * Create a new DelegatingServletInputStream.
		 * @param sourceStream the sourceStream InputStream
		 */
		public DelegatingServletInputStream(InputStream sourceStream) {
			this.sourceStream = sourceStream;
		}

		/** {@inheritDoc} */
		public int read() throws IOException {
			int value = sourceStream.read();
			++requestLength;
			return value;
		}
		/** {@inheritDoc} */
		public int available() throws IOException {
			return sourceStream.available();
		}
		/** {@inheritDoc} */
		public void close() throws IOException {
			super.close();
			sourceStream.close();
		}
		/** {@inheritDoc} */
		public void mark(int readlimit) {
			sourceStream.mark(readlimit);
		}
		/** {@inheritDoc} */
		public void reset() throws IOException {
			sourceStream.reset();
		}
		/** {@inheritDoc} */
		public boolean markSupported() {
			return sourceStream.markSupported();
		}
	}

	class DelegatingBufferedReader extends BufferedReader {
		DelegatingBufferedReader(BufferedReader sourceReader) {
			super(sourceReader, 1);// source is already buffered
		}
		/** {@inheritDoc} */
		public int read(char[] cbuf, int off, int len) throws IOException {
			int returnValue = super.read(cbuf, off, len);
			if (returnValue != -1) {
				requestLength += returnValue;
			}
			return returnValue;
		}
	}

	public MessAdminRequestWrapper(HttpServletRequest request) throws IOException {
		super(request);
	}

	/** {@inheritDoc} */
	public synchronized BufferedReader getReader() throws IOException {
		if (null == decryptedDataReader) {
			decryptedDataReader = new DelegatingBufferedReader(super.getReader());
		}
		return decryptedDataReader;
	}

	/** {@inheritDoc} */
	public synchronized ServletInputStream getInputStream() throws IOException {
		if (null == decryptedDataInputStream) {
			decryptedDataInputStream = new DelegatingServletInputStream(super.getInputStream());
		}
		return decryptedDataInputStream;
	}

	public long getRequestLength() {
		return (requestLength == -1) ? requestLength : requestLength + 1;
	}
}
